"""Constants for Roon Component."""

AUTHENTICATE_TIMEOUT = 5

DOMAIN = "roon"

DATA_CONFIGS = "roon_configs"

DEFAULT_NAME = "Roon Labs Music Player"

ROON_APPINFO = {
    "extension_id": "home_assistant",
    "display_name": "Roon Integration for Home Assistant",
    "display_version": "1.0.0",
    "publisher": "home_assistant",
    "email": "home_assistant@users.noreply.github.com",
    "website": "https://www.home-assistant.io/",
}
