"""Constants for the Gree Climate integration."""

DOMAIN = "gree"
COORDINATOR = "coordinator"

FAN_MEDIUM_LOW = "medium low"
FAN_MEDIUM_HIGH = "medium high"

MIN_TEMP = 16
MAX_TEMP = 30

MAX_ERRORS = 2

TARGET_TEMPERATURE_STEP = 1
